#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
OSD::OSD( void )
{
    /*  Init our text */
    strncpy( mText, "NULL", 64 );

    /*  Init the texture index */
    mTextureIndex = -1;

    /*  Current position */
    mPos.x = 0;
    mPos.y = 0;
}


/*
--------------------------------------------------------------------------------
                                     UPDATE
--------------------------------------------------------------------------------
*/
void OSD::update( const char *string )
{
    /*
     *  Create a color for the text, and if 'outline' is enabled, change the
     *  text color to match the outline color; otherwise it defaults to the
     *  foreground color
     */
    SDL_Color tColor = fgColor;
    if( outline )
        tColor = outlineColor;

    /*  If the string was empty, just create a texture from the prev string */
    if( string == NULL )
    {
        gTextures[ mTextureIndex ]->create_texture_from_string( gFont,
                mText, tColor );
    }

    /*  Otherwise, create a texture from the new string */
    else
    {
        strncpy( mText, string, 64 );
        gTextures[ mTextureIndex ]->create_texture_from_string( gFont,
                mText, tColor );
    }


    /*  Reset position of the OSD */
    mPos.x = ( SCREEN_WIDTH - gTextures[ mTextureIndex ]->get_width() ) / 2;
    mPos.y = 20;
}


/*
--------------------------------------------------------------------------------
                               SET_TEXTURE_INDEX
--------------------------------------------------------------------------------
*/
void OSD::set_texture_index( int textureIndex )
{
    mTextureIndex = textureIndex;
}


/*
--------------------------------------------------------------------------------
                                     RENDER
--------------------------------------------------------------------------------
*/
void OSD::render( void )
{
    /*  Grab the current alpha value of the texture and decrease that by 8 */
    Uint8 m;
    gTextures[ mTextureIndex ]->get_alpha( &m );
    m -= 8;

    /*  Set the alpha to the new value, causing it to appear to fade out */
    gTextures[ mTextureIndex ]->set_alpha( m );

    /*  Render the new, alpha-modulated texture */
    gTextures[ mTextureIndex ]->render( mPos.x, mPos.y );

    /*  Once it's faded out, disable the OSD to stop rendering */
    if( m < 8 )
        showOSD = false;
}


/*
--------------------------------------------------------------------------------
                                 POSITION STUFF
--------------------------------------------------------------------------------
*/
void OSD::set_position( int x, int y )
{
    mPos.x = x;
    mPos.y = y;
}


int OSD::get_pos_x( void )
{
    return( mPos.x );
}


int OSD::get_pos_y( void )
{
    return( mPos.y );
}
