#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
Score::Score( void )
{
    /*  The score for this score object */
    mScore = 0;

    /*  Max score */
    mMaxScore = 10;

    /*  Which paddle this is */
    mPaddle = -1;

    /*  The texture index */
    mTextureIndex = -1;

    /*  The position of the score object */
    mPos.x = 0;
    mPos.y = 0;
}



/*
--------------------------------------------------------------------------------
                                   DESTRUCTOR
--------------------------------------------------------------------------------
*/
Score::~Score( void )
{
    /*  Reset everything */
    mScore = 0;
    mTextureIndex = -1;
    mPos.x = 0;
    mPos.y = 0;
}



/*
--------------------------------------------------------------------------------
                                  SCORE STUFF
--------------------------------------------------------------------------------
*/
void Score::set_score( int s )      //  Set score
{
    /*  If we've hit max score (won game) */
    if( s >= mMaxScore )
    {
        paused = true;
        gameOver = true;

        /*  Set the screen to the game over screen */
        currentScreen = SCREEN_GAME_OVER;

        reset_game_over_screen( mPaddle );

    }

    /*  Set the score to whatever was given */
    else
    {
        mScore = s;
    }

    /*  Set the text color to the foreground color by default */
    SDL_Color tColor = fgColor;

    /*  Create score string */
    char scoreString[ 16 ];
    snprintf( scoreString, 16, "%02d", mScore );

    /*  If we're outlining, change score color to outline color */
    if( outline )
        tColor = outlineColor;

    /*  Create texture from score string */
    gTextures[ mTextureIndex ]->create_texture_from_string( gFont,
            scoreString, tColor );
}


int Score::get_score( void )        //  Get score
{
    return( mScore );
}



/*
--------------------------------------------------------------------------------
                          RENDERING AND TEXTURE INDEX
--------------------------------------------------------------------------------
*/
void Score::set_texture_index( int index )      //  Set the texture index
{
    mTextureIndex = index;
}


void Score::render( void )                      //  Render the score texture
{
    gTextures[ mTextureIndex ]->render( mPos.x, mPos.y );
}



/*
--------------------------------------------------------------------------------
                             POSITION / DIMENSIONS
--------------------------------------------------------------------------------
*/
void Score::set_position( int x, int y )    //  Set the X and Y positions
{
    mPos.x = x;
    mPos.y = y;
}

int Score::get_width( void )                //  Get the width
{
    return( gTextures[ mTextureIndex ]->get_width() );
}


int Score::get_height( void )               //  Get the height
{
    return( gTextures[ mTextureIndex ]->get_height() );
}


/*
--------------------------------------------------------------------------------
                                   SET PADDLE
--------------------------------------------------------------------------------
*/
void Score::set_paddle( int paddle )
{
    mPaddle = paddle;
}



/*
--------------------------------------------------------------------------------
                              SET / GET MAX SCORE
--------------------------------------------------------------------------------
*/
/*  Sets the maximum score */
void Score::set_max_score( int score )
{
    mMaxScore = score;
}

/*  Gets the maximum score */
int Score::get_max_score( void )
{
    return( mMaxScore );
}
