/*******************************************************************************
 *  This texture class contains a pointer to an SDL_Texture created by the
 *  create_texture_from_string method, as well as other vitals and a couple of
 *  ancillary methods for color/alpha modulation.  This texture class does not
 *  load textures from any files.
 ******************************************************************************/
#ifndef CLASS_TEXTURE_H
#define CLASS_TEXTURE_H


/*
 *  Texture class
 */
class Texture
{
    public:
        /*  Constructor */
        Texture( void );

        /*  Destructor */
        ~Texture( void );

        /*  Free texture */
        void free_texture( void );

        /*  Create a texture from a string */
        bool create_texture_from_string( TTF_Font *font, const char *string,
                SDL_Color textColor );

        /*  Set position of texture */
        void set_position( int x, int y );

        /*  Set color modulation */
        void set_color( Uint8 red, Uint8 green, Uint8 blue );

        /*  Set / get alpha modulation */
        void set_alpha( Uint8 alpha );
        void get_alpha( Uint8 *alpha );

        /*  Render the texture to the screen */
        void render( int x = 0, int y = 0 );

        /*  Get the dimensions */
        int get_width( void );
        int get_height( void );

    private:
        /*  The texture itself */
        SDL_Texture *mTexture;

        /*  The position */
        SDL_Point mPos;

        /*  The dimensions */
        int mWidth;
        int mHeight;
};

#endif
