#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                             VARIABLES AND POINTERS
--------------------------------------------------------------------------------
*/

/*  SDL Stuff */
SDL_Window *gWindow = NULL;
SDL_Renderer *gRenderer = NULL;
SDL_Joystick *pad1 = NULL;
SDL_Joystick *pad2 = NULL;
SDL_Color *colors[ TOTAL_COLORS ];
SDL_Color fgColor;
SDL_Color bgColor;
SDL_Color outlineColor;
SDL_Rect midRect;

/*  SDL_ttf stuff */
TTF_Font *gFont = NULL;
TTF_Font *gFontHelp = NULL;

/*  SDL_mixer stuff */
Mix_Chunk *gSoundScore = NULL;
Mix_Chunk *gSoundBounceWall = NULL;
Mix_Chunk *gSoundBouncePaddle = NULL;

/*  Constants */
const int SCREEN_WIDTH = 800;
const int SCREEN_HEIGHT = 600;
const int DEFAULT_FPS = 60;
const int DEADZONE = 8000;

/*  Integers */
int osdFadeTicks = 0;
int pausedTicks = 0;
int currentScreen = SCREEN_MAIN;
int valuePad1 = 0;
int valuePad2 = 0;

/*  Booleans */
bool paused = true;
bool reset = true;
bool midline = true;
bool outline = false;
bool showOSD = false;
bool showScores = true;
bool mute = false;
bool gameOver = false;
bool quit = false;



/*
--------------------------------------------------------------------------------
                              FUNCTION PROTOTYPES
--------------------------------------------------------------------------------
*/

bool init( void );
bool load_media( void );
void close( void );
bool check_collision( SDL_Rect a, SDL_Rect b );
void handle_collision( void );
bool colors_match( SDL_Color c1, SDL_Color c2 );


/*
--------------------------------------------------------------------------------
                                    MY STUFF
--------------------------------------------------------------------------------
*/

/*  Global textures arrays */
Texture *gTextures[ T_TOTAL ];
Texture *gHelpScreen[ T_HELP_TOTAL ];
Texture *gHelpScreenGamepad[ T_HELP_TOTAL_G ];
Texture *gGameOverTextures[ T_GAME_OVER_TOTAL ];

/*  Custom objects */
Ball *ball;
Paddle *paddles[ 2 ];
Score *gScores[ 2 ];
OSD osd;


/*
--------------------------------------------------------------------------------
                             RESET_GAME_OVER_SCREEN
--------------------------------------------------------------------------------
*/
void reset_game_over_screen( int paddleNum )
{

    /*  Set text color */
    SDL_Color textColor = fgColor;
    if( outline )
        textColor = outlineColor;

    /*  Create the victor string */
    char victorString[ 32 ];
    snprintf( victorString, 32, "Player %d is victorious!", paddleNum + 1 );

    gGameOverTextures[ T_GAME_OVER_VICTOR ]->create_texture_from_string(
            gFont, victorString, textColor );

    gGameOverTextures[ T_GAME_OVER_PRESS_START ]->create_texture_from_string(
            gFont, "Press START to continue", textColor );

    gGameOverTextures[ T_GAME_OVER_QUIT ]->create_texture_from_string(
            gFont, "or Q to quit", textColor );
}



/*
--------------------------------------------------------------------------------
                                   PRINT_HELP
--------------------------------------------------------------------------------
*/
void print_help( void )
{
    printf("Usage:  pongo [OPTION]\n\n");
    
    printf("This is your basic pong knockoff written in SDL2.  It makes use ");
    printf("of hardware\nrendering, so if you can't play it, that's why.\n\n");

    printf("The player (or AI) that first scores 10 points wins.  To cycle ");
    printf("the AI\ndifficulty, press the 'd' key on your keyboard; other ");
    printf("commands are shown\nby pressing the 'h' key.\n\n");

    printf("You can use the keyboard, mouse or a gamepad to play this game.\n");
    printf("If you have two gamepads, plug the second one in for 2-player ");
    printf("mode.\n\n");

    printf("Options:\n");
    printf("  -h or --help:\t\tPrint this help text\n");
    printf(" --version:\t\tPrint version and author info\n");
}



/*
--------------------------------------------------------------------------------
                                 PRINT_VERSION
--------------------------------------------------------------------------------
*/
void print_version( void )
{
    printf("pongo, version %s\n", (VERSION) );
    printf("James Hendrie <hendrie.james@gmail.com>\n");
}
