/*******************************************************************************
 *  This is where all the global stuff is defined.  I do so like to keep my
 *  main.cpp under 1000 lines.
 ******************************************************************************/
#ifndef UTIL_H
#define UTIL_H

#include <stdio.h>
#include <string.h>
#include <cmath>
#include <SDL2/SDL.h>
#include <SDL2/SDL_mixer.h>
#include <SDL2/SDL_ttf.h>

#define VERSION "0.9"


/*
--------------------------------------------------------------------------------
                                     ENUMS
--------------------------------------------------------------------------------
*/
enum colors
{
    COLOR_WHITE,
    COLOR_RED,
    COLOR_YELLOW,
    COLOR_PURPLE,
    COLOR_CYAN,
    COLOR_BLUE,
    COLOR_GREEN,
    COLOR_BLACK,
    TOTAL_COLORS
};

enum screens
{
    SCREEN_MAIN,
    SCREEN_SELECT,
    SCREEN_HELP,
    SCREEN_GAME_OVER,
    TOTAL_SCREENS
};


enum globalTextures
{
    T_SCORE1,
    T_SCORE2,
    T_OSD,
    T_TOTAL
};

enum helpScreenTextures
{
    T_HELP_KEYBOARD,
    T_HELP_GO,
    T_HELP_HELP,
    T_HELP_TOGGLE_SOUND,
    T_HELP_CYCLE_DIFFICULTY,
    T_HELP_TOGGLE_FG,
    T_HELP_TOGGLE_BG,
    T_HELP_TOGGLE_LINE,
    T_HELP_TOGGLE_SCORES,
    T_HELP_TOTAL
};

enum helpScreenTexturesGamepad
{
    T_HELP_GAMEPAD,
    T_HELP_GO_G,
    T_HELP_HELP_G,
    T_HELP_TOGGLE_SOUND_G,
    T_HELP_CYCLE_DIFFICULTY_G,
    T_HELP_TOGGLE_FG_G,
    T_HELP_TOGGLE_BG_G,
    T_HELP_TOGGLE_LINE_G,
    T_HELP_TOGGLE_SCORES_G,
    T_HELP_TOTAL_G
};

enum buttons
{
    B_A,
    B_B,
    B_X,
    B_Y,
    B_LB,
    B_RB,
    B_SELECT,
    B_START,
    B_TOTAL_BUTTONS
};

enum gameOverTextures
{
    T_GAME_OVER_VICTOR,
    T_GAME_OVER_PRESS_START,
    T_GAME_OVER_QUIT,
    T_GAME_OVER_TOTAL
};




/*
--------------------------------------------------------------------------------
                             VARIABLES AND POINTERS
--------------------------------------------------------------------------------
*/


/*  SDL Stuff */
extern SDL_Window *gWindow;                 //  Our window
extern SDL_Renderer *gRenderer;             //  Renderer
extern SDL_Joystick *pad1;                  //  Gamepad at index 0
extern SDL_Joystick *pad2;                  //  Gamepad at index 1
extern SDL_Color *colors[ TOTAL_COLORS ];   //  Commonly used colors
extern SDL_Color fgColor;                   //  Foreground color
extern SDL_Color bgColor;                   //  Background color
extern SDL_Color outlineColor;              //  Outline color
extern SDL_Rect midRect;                    //  Rect for midline 'dash'

/*  SDL_ttf stuff */
extern TTF_Font *gFont;                 //  Normal (big) sized font
extern TTF_Font *gFontHelp;             //  Smaller font

/*  SDL_mixer stuff */
extern Mix_Chunk *gSoundScore;          //  Sound when someone scores
extern Mix_Chunk *gSoundBounceWall;     //  Sound when the ball bounces off wall
extern Mix_Chunk *gSoundBouncePaddle;   //  Sound when ball bounces off paddle

/*  Constants */
extern const int SCREEN_WIDTH;          //  Screen width
extern const int SCREEN_HEIGHT;         //  Screen height
extern const int DEFAULT_FPS;           //  Default FPS (for limiting purposes)
extern const int DEADZONE;              //  Joystick deadzone

/*  Integers */
extern int pausedTicks;                 //  Ticks since game was paused
extern int osdFadeTicks;                //  Ticks since OSD shown on screen
extern int currentScreen;               //  Current screen we're on
extern int valuePad1;                   //  jaxis value for paddle 1 (left)
extern int valuePad2;                   //  jaxis value for paddle 2 (right)

/*  Booleans */
extern bool paused;                     //  Are we paused
extern bool reset;                      //  Are we resetting the ball/paddles
extern bool midline;                    //  Do we display the middle line
extern bool outline;                    //  Are we outlining foreground elements
extern bool showOSD;                    //  Are we showing the on-screen display
extern bool showScores;                 //  Do we display the scores
extern bool mute;                       //  Is the sound muted
extern bool gameOver;                   //  Whether or not a paddle has won
extern bool quit;                       //  Whether the player wants to quit



/*
--------------------------------------------------------------------------------
                              FUNCTION PROTOTYPES
--------------------------------------------------------------------------------
*/


extern bool init( void );
extern bool load_media( void );
extern void close( void );
extern bool check_collision( SDL_Rect a, SDL_Rect b );
extern void handle_collision( void );
extern bool colors_match( SDL_Color c1, SDL_Color c2 );
extern void render_midline( void );
extern void render_main( void );
extern void events_main( SDL_Event &e );
extern void change_color( SDL_Color &color );
extern void reset_game_over_screen( int paddleNum );
extern void print_help( void );
extern void print_version( void );


/*
--------------------------------------------------------------------------------
                               CLASS DEFINITIONS
--------------------------------------------------------------------------------
*/

/*
 *  We'll have to include the classes before declaring any pointers or instances
 *  derived from them.
 */
#ifndef CLASSES_H
#include "classes.h"
#endif



/*
--------------------------------------------------------------------------------
                                    MY STUFF
--------------------------------------------------------------------------------
*/

/*  Global textures arrays */
extern Texture *gTextures[ T_TOTAL ];
extern Texture *gHelpScreen[ T_HELP_TOTAL ];
extern Texture *gHelpScreenGamepad[ T_HELP_TOTAL_G ];
extern Texture *gGameOverTextures[ T_GAME_OVER_TOTAL ];

/*  Custom objects */
extern Ball *ball;
extern Paddle *paddles[ 2 ];
extern Score *gScores[ 2 ];
extern OSD osd;


#endif
